"use strict";
/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const moment_1 = require("moment");
exports.isDuration = moment_1.isDuration;
const timeFormatRegex = /^(0|[1-9][0-9]*)(ms|s|m|h|d|w|M|Y)$/;
function stringToDuration(text) {
    const result = timeFormatRegex.exec(text);
    if (!result) {
        throw new Error(`Failed to parse [${text}] as time value. ` +
            `Format must be <count>[ms|s|m|h|d|w|M|Y] (e.g. '70ms', '5s', '3d', '1Y')`);
    }
    const count = parseInt(result[1], 0);
    const unit = result[2];
    return moment_1.duration(count, unit);
}
function numberToDuration(numberMs) {
    if (!Number.isSafeInteger(numberMs) || numberMs < 0) {
        throw new Error(`Failed to parse [${numberMs}] as time value. ` +
            `Value should be a safe positive integer number.`);
    }
    return moment_1.duration(numberMs);
}
function ensureDuration(value) {
    if (typeof value === 'string') {
        return stringToDuration(value);
    }
    if (typeof value === 'number') {
        return numberToDuration(value);
    }
    return value;
}
exports.ensureDuration = ensureDuration;
