"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getByAlias = getByAlias;
/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

/**
 * This is used for looking up function/argument definitions. It looks through
 * the given object/array for a case-insensitive match, which could be either the
 * `name` itself, or something under the `aliases` property.
 */
function getByAlias(specs, name) {
  var lowerCaseName = name.toLowerCase();
  return Object.values(specs).find(function (_ref) {
    var name = _ref.name,
        aliases = _ref.aliases;

    if (name.toLowerCase() === lowerCaseName) return true;
    return (aliases || []).some(function (alias) {
      return alias.toLowerCase() === lowerCaseName;
    });
  });
}