/*
 * Licensed to Elasticsearch B.V. under one or more contributor
 * license agreements. See the NOTICE file distributed with
 * this work for additional information regarding copyright
 * ownership. Elasticsearch B.V. licenses this file to you under
 * the Apache License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import { parseInterval } from './utils/parse_interval';
import { uiModules } from './modules';
import { leastCommonInterval } from './vis/lib/least_common_interval';

uiModules
  .get('kibana')
  .directive('validateDateInterval', function () {
    return {
      restrict: 'A',
      require: 'ngModel',
      link: function ($scope, $el, attrs, ngModelCntrl) {
        const baseInterval = attrs.validateDateInterval || null;

        ngModelCntrl.$parsers.push(check);
        ngModelCntrl.$formatters.push(check);

        function check(value) {
          if(baseInterval) {
            ngModelCntrl.$setValidity('dateInterval', parseWithBase(value) === true);
          } else {
            ngModelCntrl.$setValidity('dateInterval', parseInterval(value) != null);
          }
          return value;
        }

        // When base interval is set, check for least common interval and allow
        // input the value is the same. This means that the input interval is a
        // multiple of the base interval.
        function parseWithBase(value) {
          try {
            const interval = leastCommonInterval(baseInterval, value);
            return interval === value.replace(/\s/g, '');
          } catch(e) {
            return false;
          }
        }
      }
    };
  });

